﻿using System;
using System.Collections.ObjectModel;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.WinForms;

namespace _9432
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.Meter;

            ShapeFileFeatureLayer.BuildIndexFile(@"..\..\AppData\test.shp", BuildIndexMode.DoNotRebuild);

            LayerOverlay layerOverlay = new LayerOverlay();
            ShapeFileFeatureLayer shapeFileLayer = new ShapeFileFeatureLayer(@"..\..\AppData\test.shp");
            shapeFileLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            shapeFileLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            layerOverlay.Layers.Add(shapeFileLayer);
            winformsMap1.Overlays.Add("o", layerOverlay);

            Proj4Projection proj4 = new Proj4Projection(25832, 3857);
            proj4.Open();
            shapeFileLayer.FeatureSource.Projection = proj4;

            shapeFileLayer.Open();
            winformsMap1.CurrentExtent = proj4.ConvertToExternalProjection(new RectangleShape(577566.851599043, 5754458.30981342, 577793.773459161, 5754297.97160437));
            winformsMap1.Refresh();
        }

        public void ExportBitmap(string filename, System.Drawing.Imaging.ImageFormat format, int pictureWidth = 1000, RectangleShape ExportExtent = null/* TODO Change to default(_) if this is not a reference type */, bool BackColor = false)
        {
            Bitmap bitmap = null/* TODO Change to default(_) if this is not a reference type */;

            if (ExportExtent == null)
                ExportExtent = winformsMap1.CurrentExtent;
                        
            RectangleShape MapRect = winformsMap1.CurrentExtent;

            int MapWidth = System.Convert.ToInt32(MapRect.Width);
            int MapHeight = System.Convert.ToInt32(MapRect.Height);
            if (pictureWidth > 0)
            {
                double f = pictureWidth / (double)MapRect.Width;
                MapHeight = (int)(f * MapRect.Height);
                MapWidth = pictureWidth;
            }

            bitmap = new Bitmap(MapWidth, MapHeight);

            Graphics g = Graphics.FromImage(bitmap);
            if (BackColor)
                g.FillRectangle(Brushes.White, 0, 0, MapWidth, MapHeight);

            PlatformGeoCanvas gdiPlusPlatformGeoCanvas = new PlatformGeoCanvas();

            gdiPlusPlatformGeoCanvas.BeginDrawing(bitmap, ExportExtent.GetBoundingBox(), winformsMap1.MapUnit);
            gdiPlusPlatformGeoCanvas.DrawingQuality = DrawingQuality.HighQuality;

            Collection<SimpleCandidate> labelsInAllLayers = new Collection<SimpleCandidate>();
            foreach (LayerOverlay exportOverlay in winformsMap1.Overlays)
            {
                //if (IsMapOverlayVisible(exportOverlay.Name))
                if (exportOverlay.IsVisible)
                {
                    foreach (Layer exportLayer in exportOverlay.Layers)
                    {
                        if (exportLayer.IsVisible)
                            exportLayer.Draw(gdiPlusPlatformGeoCanvas, labelsInAllLayers);
                    }
                }
            }

            gdiPlusPlatformGeoCanvas.EndDrawing();

            FileInfo MyFile = new FileInfo(filename);

            bitmap.Save(filename, format);

            StringBuilder worldFile = new StringBuilder();
            double PixelSizeX = MapRect.Width / (double)MapWidth;
            double PixelSizeY = MapRect.Height / (double)MapHeight;
            double XCoord = MapRect.UpperLeftPoint.X;
            double YCoord = MapRect.UpperLeftPoint.Y;

            worldFile.AppendLine(PixelSizeX.ToString());
            worldFile.AppendLine("0");
            worldFile.AppendLine("0");

            worldFile.AppendLine("-" + PixelSizeY.ToString());
            worldFile.AppendLine(XCoord.ToString("0.000"));
            worldFile.AppendLine(YCoord.ToString("0.000"));

            File.WriteAllText(MyFile.FullName.Replace(MyFile.Extension, ".tfw"), worldFile.ToString().Replace(",", "."), Encoding.ASCII);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            ExportBitmap(@"..\..\AppData\testResult.tif", System.Drawing.Imaging.ImageFormat.Tiff);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            LayerOverlay layerOverlay = winformsMap1.Overlays["o"] as LayerOverlay;

            ShapeFileFeatureLayer shpLayer = layerOverlay.Layers[0] as ShapeFileFeatureLayer;
            shpLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Water1;
            shpLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            if (layerOverlay.Layers.Count > 1)
            {
                layerOverlay.Layers.RemoveAt(1);
            }

            NativeImageRasterLayer tiffLayer = new NativeImageRasterLayer(@"..\..\AppData\testResult.tif", @"..\..\AppData\testResult.tfw");
            layerOverlay.Layers.Add(tiffLayer);
            tiffLayer.Open();
            
            RectangleShape rect = tiffLayer.GetBoundingBox();
            winformsMap1.CurrentExtent = rect;

            winformsMap1.Refresh();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            LayerOverlay layerOverlay = winformsMap1.Overlays["o"] as LayerOverlay;

            if (layerOverlay.Layers.Count > 1)
            {
                layerOverlay.Layers[1].IsVisible = !layerOverlay.Layers[1].IsVisible;
            }

            winformsMap1.Refresh();
        }
    }
}
